﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows.Input;
using System.Windows.Threading;

namespace HierarchyConverter.Wpf.Sample.ViewModels
{
	public class FolderViewModel : ViewModelBase, IContextMenuProvider, IHierarchyAware
	{

		public FolderViewModel()
		{
			timer.Tick += new EventHandler(timer_Tick);
		}

		void timer_Tick(object sender, EventArgs e)
		{
			this.Id = ++idCounter;
			timer.Stop();
		}

		private static int idCounter;

		private static Random generator = new Random(1231);
		public static IList<FolderViewModel> Generate()
		{
			var result = new ObservableCollection<FolderViewModel>();
			for (idCounter = 1; idCounter < 100; idCounter++)
			{
				var newModel = new FolderViewModel();
				newModel.Name = String.Format("Folder {0}", idCounter);
				newModel.Id = idCounter;
				newModel.ParentId = generator.Next(idCounter) + 1;
				result.Add(newModel);
			}

			// Add a few root items:
			for (var i = 1; i < 10; i++)
			{
				result[generator.Next(100)].ParentId = 0;
			}

			return result;
		}


		private bool isInEditMode;
		/// <summary>
		///     Gets or sets the name.
		/// </summary>
		public bool IsInEditMode
		{
			get
			{
				return this.isInEditMode;
			}
			set
			{
				this.SetProperty("IsInEditMode", ref isInEditMode, value);
				this.OnIsinEditModeChanged(value);
			}
		}

		DispatcherTimer timer = new DispatcherTimer() { Interval = TimeSpan.FromSeconds(3) };

		private void OnIsinEditModeChanged(bool value)
		{
			if (!value && IsTempId(this.Id))
			{
				//Brand new item, needs to be persisted
				timer.Start();
			}
		}


		private string name;
		/// <summary>
		///     Gets or sets the name.
		/// </summary>
		public string Name
		{
			get
			{
				return this.name;
			}
			set
			{
				this.SetProperty("Name", ref name, value);
			}
		}


		private int id;
		/// <summary>
		///     Gets or sets the id of the folder.
		/// </summary>
		public int Id
		{
			get
			{
				return this.id;
			}
			set
			{
				this.SetProperty("Id", ref id, value);
			}
		}


		private int parentId;
		/// <summary>
		///     Gets or sets the id of the parent folder.
		/// </summary>
		public int ParentId
		{
			get
			{
				return this.parentId;
			}
			set
			{
				this.SetProperty("ParentId", ref parentId, value);
			}
		}

		private bool isExpanded;
		/// <summary>
		///     Gets or sets whther the item is expanded.
		/// </summary>
		public bool IsExpanded
		{
			get
			{
				return this.isExpanded;
			}
			set
			{
				this.SetProperty("IsExpanded", ref isExpanded, value);
			}
		}



		public static DelegateCommand<FolderViewModel> Remove = new DelegateCommand<FolderViewModel>("Unload", OnRemoveExecute, null);

		private static void OnRemoveExecute(FolderViewModel folder)
		{
			folder.ParentId = -1;
		}

		public static DelegateCommand<FolderViewModel> AddChild = new DelegateCommand<FolderViewModel>("Add New Item", OnAddChildExecute, null);

		public static IList<ICommand> Commands = new List<ICommand>();

		static FolderViewModel()
		{
			Commands.Add(Remove);
			Commands.Add(AddChild);
		}

		private static int safeIdCounter = int.MaxValue - 10000;

		private static void OnAddChildExecute(FolderViewModel folder)
		{
			var child = new FolderViewModel();
			child.Id = GetTempId();
			child.Name = "New Item";
			child.IsInEditMode = true;

			folder.children.Add(child);
			folder.IsExpanded = true;
		}

		private bool IsTempId(int id)
		{
			return id >= int.MaxValue - 10000; 
		}

		private static int GetTempId()
		{
			return safeIdCounter++;
		}

		public IEnumerable<System.Windows.Input.ICommand> ProvideCommands()
		{
			return Commands;
		}

		IList<FolderViewModel> children;

		public void SetChildren(System.Collections.IEnumerable children, object converter)
		{
			this.children = children as IList<FolderViewModel>;
		}
	}
}
