﻿//-----------------------------------------------------------------------
// <copyright file="IHierarchyBaseCollection.cs" company="Telerik">
//     Copyright (c) Telerik.
// </copyright>
// <author>Miroslav Paskov</author>
//-----------------------------------------------------------------------

namespace HierarchyConverter
{
    using System.Collections.Generic;
    using System.ComponentModel;

    /// <summary>
    /// An internal interface used to identify a ReverseHierarchyConverter with just one of its generic properties.
    /// This way a more natural type with one generic parameter can be used for the node collection.
    /// </summary>
    /// <typeparam name="TItem">The type of the items used by the base collection.</typeparam>
    internal interface IHierarchyBaseCollection<TItem>
    {
        /// <summary>
        /// Called by a child collection when an the INotifyPropertyChanged event of one of its items has been called.
        /// </summary>
        /// <param name="nodeCollection">The node collection whose item has changed.</param>
        /// <param name="item">The item that has changed.</param>
        /// <param name="args">The event args of the property change.</param>
        void OnNodeCollectionItemChanged(NodeCollection<TItem> nodeCollection, TItem item, PropertyChangedEventArgs args);

        /// <summary>
        /// Called when the children collection has been cleared.
        /// </summary>
        /// <param name="nodeCollection">The collection that has been cleared.</param>
        /// <returns>Returns whetehr the base.Clear() should be called.</returns>
        bool OnNodeCollectionClearItems(NodeCollection<TItem> nodeCollection);

        /// <summary>
        /// Called when an item has been inserted in the children collection.
        /// </summary>
        /// <param name="nodeCollection">The children collection where an item has been inserted.</param>
        /// <param name="index">The index where an item has been inserted.</param>
        /// <param name="item">The item that has been inserted in the collection.</param>
        void OnNodeCollectionInsertItem(NodeCollection<TItem> nodeCollection, int index, TItem item);

        /// <summary>
        /// Called when an item has been removed from the collection.
        /// </summary>
        /// <param name="nodeCollection">The chidlren collection whoose item was removed.</param>
        /// <param name="index">The index of the removed item.</param>
        /// <param name="oldItem">The item that was removed.</param>
        void OnNodeCollectionRemoveItem(NodeCollection<TItem> nodeCollection, int index, TItem oldItem);

        /// <summary>
        /// Called when an item in the children collection has been replaced.
        /// </summary>
        /// <param name="nodeCollection">The collection where an item has been replaced.</param>
        /// <param name="index">The index of the replaced item.</param>
        /// <param name="oldItem">The item that was removed.</param>
        /// <param name="item">The item that was inserted.</param>
        void OnNodeCollectionSetItem(NodeCollection<TItem> nodeCollection, int index, TItem oldItem, TItem item);
    }
}
