﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Windows
Imports System.Windows.Input
Imports System.Windows.Controls
Imports Telerik.Windows
Imports Telerik.Windows.Controls

Public Class ContextMenuBuilder
    Public Shared BuildContextMenu As Telerik.Windows.RoutedEvent = EventManager.RegisterRoutedEvent("BuildContextMenu", RoutingStrategy.Bubble, GetType(BuildContextMenuEventHandler), GetType(ContextMenuBuilder))

    Shared Sub New()
        EventManager.RegisterClassHandler(GetType(Control), BuildContextMenu, New BuildContextMenuEventHandler(AddressOf OnControlBuildContextMenu))
    End Sub

    Private Shared Sub OnControlBuildContextMenu(ByVal sender As Object, ByVal e As BuildContextMenuEventArgs)
        Dim element = TryCast(sender, Control)
        Dim contextMenuAware = TryCast(element.DataContext, IContextMenuProvider)
        If contextMenuAware IsNot Nothing Then
            For Each command In contextMenuAware.ProvideCommands()
                If Not e.CommandsAndParameters.ContainsKey(command) Then
                    e.CommandsAndParameters(command) = contextMenuAware
                End If
            Next
        End If
    End Sub

    Friend Shared Sub OpenContextMenu(ByVal contextMenu As RadContextMenu, ByVal sourceUIElement As UIElement, ByVal p As Point)
        contextMenu.Placement = PlacementMode.Absolute
        contextMenu.PlacementRectangle = New Rect(p, New Size())
        contextMenu.Items.Clear()

        Dim args = New BuildContextMenuEventArgs(sourceUIElement)
        sourceUIElement.[RaiseEvent](args)

        Dim menuItemTemplate = TryCast(App.Current.Resources("AppContextMenuItemTemplate"), DataTemplate)

        For Each pair In args.CommandsAndParameters
            Dim menuItem = New RadMenuItem()
            menuItem.Header = pair.Key
            menuItem.CommandParameter = pair.Value
            menuItem.Command = pair.Key
            menuItem.HeaderTemplate = menuItemTemplate
            contextMenu.Items.Add(menuItem)
        Next

        contextMenu.IsOpen = True
    End Sub
End Class

Public Class BuildContextMenuEventArgs
    Inherits Telerik.Windows.RadRoutedEventArgs
    Public Sub New(ByVal source As Object)
        MyBase.New(ContextMenuBuilder.BuildContextMenu, source)
        CommandsAndParameters = New Dictionary(Of ICommand, Object)()
    End Sub

    Private _CommandsAndParameters As IDictionary(Of ICommand, Object)
    Public Property CommandsAndParameters() As IDictionary(Of ICommand, Object)
        Get
            Return _CommandsAndParameters
        End Get
        Set(ByVal value As IDictionary(Of ICommand, Object))
            _CommandsAndParameters = value
        End Set
    End Property
End Class

Public Interface IContextMenuProvider
    Function ProvideCommands() As IEnumerable(Of ICommand)
End Interface

Public Delegate Sub BuildContextMenuEventHandler(ByVal sender As Object, ByVal e As BuildContextMenuEventArgs)

Public Class DelegateCommand(Of T)
    Implements ICommand

    Private onExecute As Action(Of T)
    Private onCanExecute As Func(Of T, Boolean)

    Public Sub New(ByVal text As String, ByVal onExecute As Action(Of T), ByVal onCanExecute As Func(Of T, Boolean))
        Me.onCanExecute = onCanExecute
        Me.onExecute = onExecute
        Me.Text = text
    End Sub

    Private _Text As String
    Public Property Text() As String
        Get
            Return _Text
        End Get
        Set(ByVal value As String)
            _Text = value
        End Set
    End Property

    Public Function CanExecute(ByVal parameter As Object) As Boolean Implements ICommand.CanExecute
        If onCanExecute IsNot Nothing AndAlso TypeOf parameter Is T Then
            Return onCanExecute(DirectCast(parameter, T))
        End If
        Return onCanExecute Is Nothing
    End Function

    Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

    Public Sub Execute(ByVal parameter As Object) Implements ICommand.Execute
        If onExecute IsNot Nothing AndAlso TypeOf parameter Is T Then
            onExecute(DirectCast(parameter, T))
        End If
    End Sub

End Class
