﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Collections.ObjectModel
Imports System.Windows.Input
Imports HierarchyConverter

Public Class FolderViewModel
    Inherits ViewModelBase
    Implements IContextMenuProvider
    Implements IHierarchyAware

    Private Shared generator As New Random(1231)
    Public Shared Function Generate() As IList(Of FolderViewModel)
        Dim result = New ObservableCollection(Of FolderViewModel)()
        For index = 1 To 99
            Dim newModel = New FolderViewModel()
            newModel.Name = [String].Format("Folder {0}", index)
            newModel.Id = index
            newModel.ParentId = generator.[Next](index) + 1
            result.Add(newModel)
        Next

        ' Add a few root items:
        For i = 1 To 9
            result(generator.[Next](100)).ParentId = 0
        Next

        Return result
    End Function


    Private m_isInEditMode As Boolean
    ''' <summary>
    ''' Gets or sets the name.
    ''' </summary>
    Public Property IsInEditMode() As Boolean
        Get
            Return Me.m_isInEditMode
        End Get
        Set(ByVal value As Boolean)
            Me.SetProperty("IsInEditMode", m_isInEditMode, value)
        End Set
    End Property


    Private m_name As String
    ''' <summary>
    ''' Gets or sets the name.
    ''' </summary>
    Public Property Name() As String
        Get
            Return Me.m_name
        End Get
        Set(ByVal value As String)
            Me.SetProperty("Name", m_name, value)
        End Set
    End Property


    Private m_id As Integer
    ''' <summary>
    ''' Gets or sets the id of the folder.
    ''' </summary>
    Public Property Id() As Integer
        Get
            Return Me.m_id
        End Get
        Set(ByVal value As Integer)
            Me.SetProperty("Id", m_id, value)
        End Set
    End Property


    Private m_parentId As Integer
    ''' <summary>
    ''' Gets or sets the id of the parent folder.
    ''' </summary>
    Public Property ParentId() As Integer
        Get
            Return Me.m_parentId
        End Get
        Set(ByVal value As Integer)
            Me.SetProperty("ParentId", m_parentId, value)
        End Set
    End Property

    Private m_isExpanded As Boolean
    ''' <summary>
    ''' Gets or sets whther the item is expanded.
    ''' </summary>
    Public Property IsExpanded() As Boolean
        Get
            Return Me.m_isExpanded
        End Get
        Set(ByVal value As Boolean)
            Me.SetProperty("IsExpanded", m_isExpanded, value)
        End Set
    End Property



    Public Shared Remove As New DelegateCommand(Of FolderViewModel)("Unload", AddressOf OnRemoveExecute, Nothing)

    Private Shared Sub OnRemoveExecute(ByVal folder As FolderViewModel)
        folder.ParentId = -1
    End Sub

    Public Shared AddChild As New DelegateCommand(Of FolderViewModel)("Add New Item", AddressOf OnAddChildExecute, Nothing)

    Public Shared Commands As IList(Of ICommand) = New List(Of ICommand)()

    Shared Sub New()
        Commands.Add(Remove)
        Commands.Add(AddChild)
    End Sub

    Private Shared Sub OnAddChildExecute(ByVal folder As FolderViewModel)
        Dim child = New FolderViewModel()
        child.Id = generator.[Next](12123)
        child.Name = "New Item"
        child.IsInEditMode = True

        folder.children.Add(child)
        folder.IsExpanded = True
    End Sub

    Public Function ProvideCommands() As IEnumerable(Of System.Windows.Input.ICommand) Implements IContextMenuProvider.ProvideCommands
        Return Commands
    End Function

    Private children As IList(Of FolderViewModel)

    Public Sub SetChildren(ByVal children As System.Collections.IEnumerable, ByVal converter As Object) Implements IHierarchyAware.SetChildren
        Me.children = TryCast(children, IList(Of FolderViewModel))
    End Sub

End Class
