﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using HierarchyConverter.Wpf.Sample;
using System.Collections.Generic;

namespace TreeViewContextMenu.ViewModels
{
	/// <summary>
	/// This class will represent a leaf in our hierarchical structure.
	/// </summary>
	public class FileViewModel : DiskItemBase , IContextMenuProvider
	{
		private static readonly string[] nouns;
		private static readonly string[] adjectives;
		private static readonly string[] firstNames;
		private static readonly string[] lastNames;
		private string name;
		private string note;
		private int size;
		private bool isPublic;

		private static string[] Split(string source)
		{
			return source.Split(new char[] { ' ', '\n', '\t', '\r' }, StringSplitOptions.RemoveEmptyEntries);
		}

		static FileViewModel()
		{
			lastNames = Split(@"Brown Smith Patel Jones Williams Johnson Taylor Thomas Roberts 
			Khan Lewis Jackson Clarke James Phillips Wilson Ali Mason Mitchell Rose 
			Davis Davies Rodríguez Cox Alexander");

			firstNames = Split(@"Jack William Lachlan Joshua Riley Thomas Cooper Oliver James Ethan
			Mia Chloe Isabella Charlotte Emily Ella Olivia Sienna Ava Sophie
			Eetu Veeti Aleksi Elias Joona Onni Arttu Leevi Juho Lauri
			Enzo Mathis Lucas Hugo Mathéo Nathan Théo Noah Mattéo Thomas");

			nouns = Split(@"contract plan permission certificate design novel company");

			adjectives = Split(@"Draft Secret Extraordinary House Final Proposed");
		}

		public FileViewModel()
		{
			this.Uid = FolderViewModel.UidCount++;
			this.Size = FolderViewModel.Generator.Next(10000);
		}

		public int Size
		{
			get
			{
				return this.size;
			}
			set
			{
				this.SetProperty("Size", ref size, value);
			}
		}

		public bool IsPublic
		{
			get
			{
				return this.isPublic;
			}
			set
			{
				this.SetProperty("IsPublic", ref isPublic, value);
			}
		}

		public override string TypeName
		{
			get
			{
				return "Word Document";
			}
		}

		public override string Name
		{
			get
			{
				if (name == null)
				{
					name = this.BuildName();
				}
				return name;
			}
			set
			{
				this.ValidateName(value);
				this.SetProperty("Name", ref name, value);
			}
		}

		public int Uid { get; set; }

		private string BuildName()
		{
			this.AuthorName = string.Format("{1} {0}", firstNames[FolderViewModel.Generator.Next(firstNames.Length)], lastNames[FolderViewModel.Generator.Next(lastNames.Length)]);
			return string.Format("{1} {0}.docx", nouns[FolderViewModel.Generator.Next(nouns.Length)], adjectives[FolderViewModel.Generator.Next(adjectives.Length)]);
		}

		private string authorName;
		public string AuthorName
		{
			get
			{
				return this.authorName;
			}
			set
			{
				this.SetProperty("AuthorName", ref authorName, value);
			}
		}

		public string Note
		{
			get
			{
				return this.note;
			}
			set
			{
				if (String.IsNullOrEmpty(value))
				{
					this.note = String.Empty;
				}
				else
				{
					this.note = String.Format("Note: {0}", value);
				}
				this.OnPropertyChanged("Note");
			}
		}

		public static ICommand RenameFileCommand = new DelegateCommand<FileViewModel>(
			"Rename",
			f => f.IsEdited = true,
			null);

		public IEnumerable<ICommand> ProvideCommands()
		{
			yield return RenameFileCommand;
		}
	}
}
