﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using HierarchyConverter;
using HierarchyConverter.Wpf.Sample.ViewModels;

namespace TreeViewSearch
{
	public partial class MainPage : UserControl
	{
		FolderConverter hierarchy;
		private int skipCount;
		private string searchText;

		public MainPage()
		{
			InitializeComponent();

			hierarchy = this.Resources["folderHierarchyConverter"] as FolderConverter;

			hierarchy.ItemsSource = FolderViewModel.Generate();

		}

		private void OnSearchTextChanged(object sender, TextChangedEventArgs e)
		{
			var textBox = sender as TextBox;

			skipCount = 0;
			searchText = textBox.Text;

			this.SearchForItem();
		}

		private void SearchForItem()
		{
			if (string.IsNullOrEmpty(searchText))
			{
				return;
			}

			searchText = searchText.ToLower();

			var matches = hierarchy.ItemsSource.Where(item => item.Name.ToLower().StartsWith(searchText));

			var count = matches.Count();

			var match = matches.Skip(skipCount).FirstOrDefault();

			searchResults.Text = String.Format("Found {0} mathces, viewing {1}th.", count, skipCount + 1);

			if (match != null)
			{
				var path = hierarchy.GetItemIdPath(match);
				var treeItem = treeView.GetItemByPath(path);

				treeView.ExpandItemByPath(path);

				treeItem.IsSelected = true;
			}
		}

		private void OnPrevClick(object sender, RoutedEventArgs e)
		{
			skipCount = Math.Max(skipCount - 1, 0);
			SearchForItem();
		}

		private void OnNextClick(object sender, RoutedEventArgs e)
		{
			skipCount++;
			SearchForItem();
		}
	}

	public class FolderConverter : ReverseHierarchyConverter<FolderViewModel, int>
	{
		public FolderConverter()
		{
			this.Initialize(f => f.Id, f => f.ParentId, 0, -1);
		}
	}
}
